unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, IniFiles, StdCtrls, FileCtrl, ComCtrls, ExtCtrls,
  ExtDlgs, Buttons,
  AviHandler;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    StatusBar: TStatusBar;
    DriveComboBox: TDriveComboBox;
    DirectoryListBox: TDirectoryListBox;
    FileListBox: TFileListBox;
    Panel2: TPanel;
    ScrollBox1: TScrollBox;
    Image: TImage;
    FileName: TStaticText;
    SavePictureDialog: TSavePictureDialog;
    Timer1: TTimer;
    PageMode: TPageControl;
    TabPlay: TTabSheet;
    TabRec: TTabSheet;
    ListCodecs: TListBox;
    TrackBar1: TTrackBar;
    ButtonPlay: TButton;
    ButtonStop: TButton;
    Label1: TLabel;
    Label2: TLabel;
    ButtonSave: TButton;
    ProgConv: TProgressBar;
    ButtonConv: TButton;
    SaveAVIDlg: TSaveDialog;
    Label3: TLabel;
    ButtonStopConv: TButton;
    TrackQual: TTrackBar;
    Label4: TLabel;
    ButtonConfig: TButton;
    Label5: TLabel;
    procedure TrackBar1Change(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ButtonSaveClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure ButtonPlayClick(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure FileListBoxChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PageModeChange(Sender: TObject);
    procedure ButtonConvClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure ButtonStopConvClick(Sender: TObject);
    procedure ButtonConfigClick(Sender: TObject);
    procedure ListCodecsClick(Sender: TObject);
  private
    FPlayer : TAviPlayer;
    FRecorder : TAviRecorder;
    FFileChanged, FBreakConv : Boolean;
  protected
    procedure OpenFile(const FName: String);
    procedure PrepareTrackBar;
  end;

var
   Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormShow(Sender: TObject);
begin
FPlayer := TAviPlayer.Create; FFileChanged := True;
//OpenFile('D:\main\my_samples\DShow\DirectShowCapture\06-11-17_14-12-42.avi');
//OpenFile('D:\main\my_samples\DShow\DirectShowCapture\06-11-17_14-09-16.avi');
//OpenFile('D:\main\libs\GifImage\Demo\avi2gif\160.avi');
end;


procedure TForm1.FormHide(Sender: TObject);
begin
FRecorder.Free;
FPlayer.Free;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
If (Key = 27) then begin FBreakConv := True; Close; end;
end;

procedure TForm1.OpenFile(Const FName : String);
begin
If (FName = '') then Exit;
If FPlayer.Open(FName) then
  PrepareTrackBar;
Filename.Caption := ExtractFileName(FName) + ' (' + FPlayer.Codec + ')';
end;


procedure TForm1.PrepareTrackBar;
begin
TrackBar1.Max := FPlayer.FrameCnt - 1;
TrackBar1.Position := 0;
TrackBar1.OnChange(self);
FFileChanged := True;
end;


procedure TForm1.TrackBar1Change(Sender: TObject);
Var i : Integer;
begin
If FPlayer.Processing then begin
  Label5.Caption := Format('Video size: %d * %d; Total frames: %d',
                           [FPlayer.FrameSize.x, FPlayer.FrameSize.y, FPlayer.FrameCnt]);
  i := TrackBar1.Position;
  If FPlayer.GetFrame(Image.Picture.Bitmap, i) then begin
    Label1.Caption := 'Frame: ' + IntToStr(i);
    Label2.Caption := 'Time: ' + IntToStr(i div FPlayer.FPS) + ' s';
    Image.Repaint;
  end else
    Label1.Caption := 'Frame: ' + IntToStr(i) + ' failed';
end;
end;



procedure TForm1.ButtonSaveClick(Sender: TObject);
var Name : String;
begin
If (not FPlayer.Processing) then Exit;

SavePictureDialog.InitialDir := ExtractFilePath(ParamStr(0));
Name := ExtractFileName(FPlayer.Filename);
Delete(Name,Length(Name)-3,4);
SavePictureDialog.FileName := Name;

If SavePictureDialog.Execute then begin
  Image.Picture.SaveToFile(SavePictureDialog.FileName);
end;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
If TrackBar1.Position < TrackBar1.Max then
  TrackBar1.Position := TrackBar1.Position + 1
else begin
  ButtonStopClick(nil);
end;
end;

procedure TForm1.ButtonPlayClick(Sender: TObject);
begin
If FPlayer.Processing then begin
  If TrackBar1.Position = TrackBar1.Max then TrackBar1.Position := 0;
  Timer1.Interval := Round(1000 / FPlayer.FPS);
  Timer1.Enabled := True;
  ButtonPlay.Enabled := False; ButtonStop.Enabled := True;
end;
end;

procedure TForm1.ButtonStopClick(Sender: TObject);
begin
Timer1.Enabled := False;
ButtonPlay.Enabled := FPlayer.Processing; ButtonStop.Enabled := False;
end;

procedure TForm1.FileListBoxChange(Sender: TObject);
begin
PageMode.ActivePageIndex := 0;
OpenFile(FileListBox.FileName);
end;

procedure TForm1.PageModeChange(Sender: TObject);
begin
If (PageMode.ActivePageIndex = 1) and (FPlayer.Processing) then begin
  If (FRecorder = nil) then FRecorder := TAviRecorder.Create;
  If FFileChanged then begin
    FPlayer.GetFrame(Image.Picture.Bitmap, 0);
    ButtonConv.Enabled := GetCodecList(Image.Picture.Bitmap, ListCodecs.Items) > 0;
    If ButtonConv.Enabled then begin
      ListCodecs.ItemIndex := 0; ListCodecsClick(nil);
    end;
    FFileChanged := False;
  end;
end;
end;


procedure TForm1.ButtonConvClick(Sender: TObject);
Var n, i : Integer;
begin
FBreakConv := False;
SaveAVIDlg.InitialDir := ExtractFilePath(ParamStr(0));
SaveAVIDlg.FileName := ExtractFileName( FPlayer.Filename );
If SaveAVIDlg.Execute then begin
  i := Round((TrackQual.Position / TrackQual.Max) * 10000);
  FRecorder.FPS := FPlayer.FPS;
  If FRecorder.StartRecord(SaveAVIDlg.FileName, Image.Picture.Bitmap, i) then begin

    Label3.Caption := 'Processing...';
    ButtonConv.Enabled := False; ButtonStopConv.Enabled := True;
    ProgConv.Position := 0;
    i := FPlayer.FrameCnt div ProgConv.Max; If (i = 0) then i := 1;

    For n:=0 to FPlayer.FrameCnt-1 do
      If FPlayer.GetFrame(Image.Picture.Bitmap, n) then begin
        FRecorder.AddFrame(Image.Picture.Bitmap);
        If (n mod i) = 0 then begin
          ProgConv.Position := ProgConv.Position + 1;
          Application.ProcessMessages;
          If (FBreakConv) then Break;
        end;
      end else
        Break;

    FRecorder.Close; ProgConv.Position := ProgConv.Max;
    Label3.Caption := 'Done.';
    ButtonConv.Enabled := True; ButtonStopConv.Enabled := False;
  end;
end;
end;

procedure TForm1.ButtonStopConvClick(Sender: TObject);
begin
FBreakConv := True;
end;

procedure TForm1.ButtonConfigClick(Sender: TObject);
begin
ShowCodecConfig(Handle, FRecorder.Codec);
end;

procedure TForm1.ListCodecsClick(Sender: TObject);
Var i : Integer;
    s : String[4];
    c : TFourCC;
begin
i := ListCodecs.ItemIndex;
If (i <> -1) then begin
  s := Copy(ListCodecs.Items[i], 1, 4); Move(s[1], c, 4);
  FRecorder.Codec := c;
  ButtonConfig.Enabled := ShowCodecConfig(INVALID_HANDLE_VALUE, c);
end;
end;

end.
